<?php

namespace WDB\WdbLanguageHook\Domain\Repository;

/**
 * This file is part of the "wdb_language_hook" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\Restriction\FrontendRestrictionContainer;
use TYPO3\CMS\Core\Utility\DebugUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

/**
 * Need for this class is based on https://forge.typo3.org/issues/86762
 *
 * @author David Bruchmann <david.bruchmann@gmail.com>
 * @copyright 2019 David Bruchmann <david.bruchmann@gmail.com>
 */
class ContentRepository # extends \TYPO3\CMS\Extbase\Persistence\Repository
// implements \TYPO3\CMS\Frontend\Page\PageRepositoryGetRecordOverlayHookInterface
{

    public function getOverlayRow($table, $uid, $pid, $languageChain)
    {
        // $languageChain never exists for (some ?) BE Tables like be_users
        #if(strlen($languageChain)) {
            $tableControl  = $GLOBALS['TCA'][$table]['ctrl'] ?? [];
            $languageField = $tableControl['languageField'];
            $transOrigPointerField = $tableControl['transOrigPointerField'];
            //$languageChain = $this->getFallbackChain($parent);
            
            
        /* // EXTBASE
        $query = $this->persistenceManager->createQueryForType($this->objectType);
        if ($this->defaultOrderings !== []) {
            $query->setOrderings($this->defaultOrderings);
        }
        if ($this->defaultQuerySettings !== null) {
            $query->setQuerySettings(clone $this->defaultQuerySettings);
        }
        return $query;
        */

            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
                ->getQueryBuilderForTable($table);
            $queryBuilder->setRestrictions(
                GeneralUtility::makeInstance(FrontendRestrictionContainer::class)
            );
            $query = $queryBuilder->select('*')
                ->from($table)
                ->where(
                    $queryBuilder->expr()->eq(
                        'pid',
                        $queryBuilder->createNamedParameter($row['pid'], \PDO::PARAM_INT)
                    ),
                    $queryBuilder->expr()->in(
                        $languageField,
                        $queryBuilder->createNamedParameter(GeneralUtility::intExplode(',', $languageChain), Connection::PARAM_INT_ARRAY)
                    ),
                    $queryBuilder->expr()->eq(
                        $transOrigPointerField,
                        $queryBuilder->createNamedParameter($row['uid'], \PDO::PARAM_INT)
                    )
                )
                ->add('orderBy', 'FIELD(' . $languageField . ', ' . $languageChain . ')', true)
                ->setMaxResults(1);
            $resource = $query->execute();
            $overlayRow = $resource->fetch();
DebuggerUtility::var_dump(['$overlayRow'=>$overlayRow,'$pid'=>$pid,'$resource'=>$resource,'$languageChain'=>$languageChain,'$sql'=>$query->getSql()],__METHOD__.':'.__LINE__);
            return $overlayRow;
        #}
    }

    public function getRowByUid($table, $uid, $pid, $languageChain)
    {
        $tableControl  = $GLOBALS['TCA'][$table]['ctrl'] ?? [];
        $languageField = $tableControl['languageField'];

        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
            ->getQueryBuilderForTable($table);
        $queryBuilder->setRestrictions(
            GeneralUtility::makeInstance(FrontendRestrictionContainer::class)
        );
        $query = $queryBuilder->select('*')
            ->from($table)
            ->where(
                $queryBuilder->expr()->eq(
                    'pid',
                    $queryBuilder->createNamedParameter($row['pid'], \PDO::PARAM_INT)
                ),
                $queryBuilder->expr()->in(
                    $languageField,
                    $queryBuilder->createNamedParameter(GeneralUtility::intExplode(',', $languageChain), Connection::PARAM_INT_ARRAY)
                ),
                $queryBuilder->expr()->eq(
                    'uid',
                    $queryBuilder->createNamedParameter($row['uid'], \PDO::PARAM_INT)
                )
            );
        $resource = $query->execute();
        $row = $resource->fetch();
        return $row;
    }
}
