<?php

namespace WDB\WdbLanguageHook\Hooks\Frontend\Page;

/**
 * This file is part of the "wdb_language_hook" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\Restriction\FrontendRestrictionContainer;
use TYPO3\CMS\Core\Utility\DebugUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;

use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;

use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

use WDB\WdbLanguageHook\Domain\Repository\ContentRepository;

/**
 * Need for this class is based on https://forge.typo3.org/issues/86762
 *
 * @author David Bruchmann <david.bruchmann@gmail.com>
 * @copyright 2019 David Bruchmann <david.bruchmann@gmail.com>
 */
class PageRepository implements \TYPO3\CMS\Frontend\Page\PageRepositoryGetRecordOverlayHookInterface
{

    protected function initTsfe(){
        if (!isset($GLOBALS['TSFE'])) {
           $id       = GeneralUtility::_GP('id')       ? GeneralUtility::_GP('id')       : 0;
           $type     = GeneralUtility::_GP('type')     ? GeneralUtility::_GP('type')     : 0;
           $no_cache = GeneralUtility::_GP('no_cache') ? GeneralUtility::_GP('no_cache') : 0;
           $cHash    = GeneralUtility::_GP('cHash')    ? GeneralUtility::_GP('cHash')    : '';
           $MP       = GeneralUtility::_GP('MP');
           $GLOBALS['TSFE'] = GeneralUtility::makeInstance(TypoScriptFrontendController::class, null, $id, $type, $no_cache, $cHash, null, $MP);
        }
    }

    protected function getIntrinsicFields($config, $table)
    {
        $intrinsicFields = [];
        if (isset($config)) {
            foreach($config as $plugin => $pluginConfig) {
                if (isset($pluginConfig['intrinsicFields'][$table])) {
                    if(is_array($pluginConfig['intrinsicFields'][$table]) && count($pluginConfig['intrinsicFields'][$table])) {
                        $intrinsicFields = $pluginConfig['intrinsicFields'][$table];
                    }
                    break;
                }
            }
        }
        return count($intrinsicFields) ? $intrinsicFields : false;
    }

    /*
    protected function getRelatedTables($config, $table)
    {
        $relatedTables = [];
        if (isset($config)) {
            foreach($config as $plugin => $pluginConfig) {
                if (isset($pluginConfig['intrinsicFields'][$table])) {
                    if(is_array($pluginConfig['intrinsicFields'][$table]) && count($pluginConfig['intrinsicFields'][$table])) {
                        $intrinsicFields = $pluginConfig['intrinsicFields'][$table];
                    }
                    break;
                }
            }
        }
        return count($relatedTables) ? $relatedTables : false;
    }
    */

    protected function overlayItrinsicFields($originalRow, $overlayRow, $table)
    {
        $this->initTsfe();
        $config = GeneralUtility::removeDotsFromTS($GLOBALS['TSFE']->config['config']); //['intrinsicFields.']);
        if (isset($config['plugin'])) {
            if($intrinsicFields = $this->getIntrinsicFields($config['plugin'], $table)) {
                foreach ($intrinsicFields as $count => $fieldname) {
                    $overlayRow[$fieldname] = $originalRow[$fieldname];
                }
            }
        }
        return $overlayRow;
    }
    
    protected function getPluginForTable($table)
    {
        $this->initTsfe();
        $config = GeneralUtility::removeDotsFromTS($GLOBALS['TSFE']->config['config']); //['intrinsicFields.']);
        $pluginForTable = null;
        if (isset($config['plugin'])) {
            foreach ($config['plugin'] as $plugin => $pluginConfig) {
                if(isset($pluginConfig['relatedTables']) && is_array($pluginConfig['relatedTables']) && count($pluginConfig['relatedTables'])) {
                    foreach ($pluginConfig['relatedTables'] as $count => $relatedTable) {
                        if($table == $relatedTable) {
                            return $plugin;
                            break;
                        }
                    }
                }
            }
        }
        return $pluginForTable;
    }

    public function getRecordOverlay_preProcess($table, &$row, &$sys_language_content, $OLmode, \TYPO3\CMS\Frontend\Page\PageRepository $parent)
    {
        $tableControl  = $GLOBALS['TCA'][$table]['ctrl'] ?? [];
        $languageField = $tableControl['languageField'];
        $transOrigPointerField = $tableControl['transOrigPointerField'];
        $languageChain = $this->getFallbackChain($parent);

        # $contentRepository = GeneralUtility::makeInstance(\WDB\WdbLanguageHook\Domain\Repository\ContentRepository::class);

        if(strpos($table, '_domain_model_')) {
            // extbase extension
            // trash incoming row as extbase 
            // - did the overlay already by itself
            // - might have replaced some data with wrong values
            
            
            // #################################################################
            // query to check if parent tt_content for language is hidden
            // #################################################################
            if($listTypeForTable = $this->getPluginForTable($table)) {
                // #######
                // QUERY 1
                // #######
                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
                    ->getQueryBuilderForTable('tt_content');
                $queryBuilder->setRestrictions(
                    GeneralUtility::makeInstance(FrontendRestrictionContainer::class)
                );
                $query = $queryBuilder->select('sys_language_uid')
                    ->from('tt_content')
                    ->where(
                        $queryBuilder->expr()->in(
                            $languageField,
                            $queryBuilder->createNamedParameter(GeneralUtility::intExplode(',', $languageChain), Connection::PARAM_INT_ARRAY)
                        ),
                        $queryBuilder->expr()->eq(
                            'list_type',
                            $queryBuilder->createNamedParameter($listTypeForTable, \PDO::PARAM_STR)
                        )
                    )
                    ->add('orderBy', 'FIELD(' . $languageField . ', ' . $languageChain . ')', true);
                $resource = $query->execute();
                $results = $resource->fetchAll();
                $availableLanguages = [];
                if ($results && count($results)) {
                    foreach ($results as $count => $result) {
                        $availableLanguages[] = $result['sys_language_uid'];
                    }
                }
                $languageChain = implode(',', $availableLanguages);
            }
/*
DebuggerUtility::var_dump([
    '$listTypeForTable' => $listTypeForTable,
    '$table' => $table,
    '$availableLanguages' => $availableLanguages,
    '$languageChain' => $languageChain,
    '$sql' => $query->getSql()
    #'backtrace' => [$debug_backtrace[0],$debug_backtrace[1],$debug_backtrace[2],$debug_backtrace[3],$debug_backtrace[4],$debug_backtrace[5],],
  ],
  __METHOD__.':'.__LINE__
);
*/

            // #######
            // QUERY 2
            // #######
            # $originalRow = $contentRepository->getRowByUid($table, intval($row['uid']), intval($row['pid']), $languageChain);
            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
                ->getQueryBuilderForTable($table);
            $queryBuilder->setRestrictions(
                GeneralUtility::makeInstance(FrontendRestrictionContainer::class)
            );
            $query = $queryBuilder->select('*')
                ->from($table)
                ->where(
                    $queryBuilder->expr()->eq(
                        'pid',
                        $queryBuilder->createNamedParameter($row['pid'], \PDO::PARAM_INT)
                    ),
                    $queryBuilder->expr()->eq(
                        'uid',
                        $queryBuilder->createNamedParameter($row['uid'], \PDO::PARAM_INT)
                    ),
                    $queryBuilder->expr()->eq(
                        'hidden',
                        $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
                    )
                );
            $resource = $query->execute();
            $originalRow = $resource->fetch();

            
            // #######
            // QUERY 3
            // #######
            if($languageField && strlen($languageChain)) {
                # $overlayRow  = $contentRepository->getOverlayRow($table, intval($row['uid']), intval($row['pid']), $languageChain);
                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
                    ->getQueryBuilderForTable($table);
                $queryBuilder->setRestrictions(
                    GeneralUtility::makeInstance(FrontendRestrictionContainer::class)
                );
                $query = $queryBuilder->select('*')
                    ->from($table)
                    ->where(
                        $queryBuilder->expr()->eq(
                            'pid',
                            $queryBuilder->createNamedParameter($row['pid'], \PDO::PARAM_INT)
                        ),
                        $queryBuilder->expr()->in(
                            $languageField,
                            $queryBuilder->createNamedParameter(GeneralUtility::intExplode(',', $languageChain), Connection::PARAM_INT_ARRAY)
                        ),
                        $queryBuilder->expr()->eq(
                            $transOrigPointerField,
                            $queryBuilder->createNamedParameter($row['uid'], \PDO::PARAM_INT)
                        ),
                        $queryBuilder->expr()->eq(
                            'hidden',
                            $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
                        )
                    )
                    ->add('orderBy', 'FIELD(' . $languageField . ', ' . $languageChain . ')', true)
                    ->setMaxResults(1);
                $resource = $query->execute();
                $overlayRow = $resource->fetch();
            } else {
                $overlayRow = null;
            }
            
                                    
            #if ($originalRow[$languageField] === $parent->sys_language_uid) {
            #    // record is already in correct language
            #    return;
            #}
        }
        else {  /// if($table === 'tt_content') {

            // returning in original language
            // in Backend some variables are empty, might depend on tables
            if ($languageField && $row[$languageField] === $parent->sys_language_uid) {
                // record is already in correct language
                if ($row[$languageField]=='0') {
                    return; 
                } else {                
                    // TODO: add option for intrinsic fields (like tx_powermail_domain_model_fields.marker)
                    return;
                }
            }
            // $overlayRow  = $contentRepository->getOverlayRow($table, $row['uid'], $row['pid'], $languageChain);
            $originalRow = $row;
            
            // in Backend some variables are empty, might depend on tables
            if($languageField && strlen($languageChain)) {
                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
                    ->getQueryBuilderForTable($table);
                $queryBuilder->setRestrictions(
                    GeneralUtility::makeInstance(FrontendRestrictionContainer::class)
                );
                $query = $queryBuilder->select('*')
                    ->from($table)
                    ->where(
                        $queryBuilder->expr()->eq(
                            'pid',
                            $queryBuilder->createNamedParameter($row['pid'], \PDO::PARAM_INT)
                        ),
                        $queryBuilder->expr()->in(
                            $languageField,
                            $queryBuilder->createNamedParameter(GeneralUtility::intExplode(',', $languageChain), Connection::PARAM_INT_ARRAY)
                        ),
                        $queryBuilder->expr()->eq(
                            $transOrigPointerField,
                            $queryBuilder->createNamedParameter($row['uid'], \PDO::PARAM_INT)
                        ),
                        $queryBuilder->expr()->eq(
                            'hidden',
                            $queryBuilder->createNamedParameter(0, \PDO::PARAM_INT)
                        )

                    )
                    ->add('orderBy', 'FIELD(' . $languageField . ', ' . $languageChain . ')', true)
                    ->setMaxResults(1);
                $resource = $query->execute();
                $overlayRow = $resource->fetch();
            }
            #else {
            #    $overlayRow = null;
            #}
        }
        
        #$row = $overlayRow;

        if($languageField) {
            /*
            if(!in_array($row[$languageField], explode(',',$languageChain))){
                 #unset($row);
                 $row = [];
                 $sys_language_content = 0;
            }
            else
            */
            if($overlayRow && $row[$languageField] !== $overlayRow[$languageField]){
                $row = $overlayRow;
                $sys_language_content = $row[ $languageField ];
            }
            $overlayRow  = $this->overlayItrinsicFields($originalRow, $overlayRow, $table);
        }
        
#if($table === 'tt_content' && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']){ // && strpos($table, '_domain_model_')) { // && $row['uid']==254){
// $debug_backtrace = debug_backtrace();
// ### backtrace is enough to see where PageRepository is called, if much larger then memory is running out
DebuggerUtility::var_dump([
    '$table' => $table,
    '$row' => $row,
    '$originalRow' => $originalRow,
    'overlayRow' => $overlayRow,
    '$sys_language_content' => $sys_language_content,
    '$row['.$languageField.']' => $row[$languageField],
    '$parent->sys_language_uid' => $parent->sys_language_uid,
    '$parent' => $parent,
    #'backtrace' => [$debug_backtrace[0],$debug_backtrace[1],$debug_backtrace[2],$debug_backtrace[3],$debug_backtrace[4],$debug_backtrace[5],],
  ],
  __METHOD__.':'.__LINE__
);
#}

        // extbase is using a dirty trick (for EXT:news among others perhaps) in
        // TYPO3\CMS\Extbase\Persistence\Generic\Storage\Typo3DbBackend::overlayLanguageAndWorkspace
        // it overwrites (always temporary) sys_language_field in a row with 0:
        // $row[$GLOBALS['TCA'][$tableName]['ctrl']['languageField']] = 0;
        // therefore the strange 2nd part of the condition:
        //     ...  || ($table !== 'tt_content' && strpos($table, '_domain_model_'))
        //
        //if ($sys_language_content !== 0 || ($table !== 'tt_content' && strpos($table, '_domain_model_') && !strpos($table, 'powermail') && strpos($table, 'sys_file') !== 0)) {
        // OK if ($sys_language_content !== 0 && ($table === 'tt_content' || strpos($table, 'tx_news_') === 0)) {
        #if (
            #$sys_language_content !== 0
            #&& (
                #$table === 'tt_content'
                #|| 
                #strpos($table, '_domain_model_')
                
                //&& !strpos($table, '_powermail')
                    
                #|| $table === 'sys_file_reference'
            #)
        #) {
            
            
            #if ($row[$languageField] === $parent->sys_language_uid) {
                // record is already in correct language
            #    return;
            #}
            
            #if($row['uid'] > 0) {

                

    // if($table === 'tt_content' && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']) 
    //   DebuggerUtility::var_dump(['$table' => $table, '$sys_language_content' => $sys_language_content, '$parent->sys_language_uid' => $parent->sys_language_uid, '$languageChain' => $languageChain ]);
         

    // tt_content
    // if($table === 'tx_news_domain_model_news' && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']) DebuggerUtility::var_dump(['$table' => $table, '$sys_language_content' => $sys_language_content, '$parent->sys_language_uid' => $parent->sys_language_uid, '$languageChain' => $languageChain, '$row'=> $row, '$olrow'=>$olrow ]);
                    
                /*
                #if($table === 'sys_file_reference' || strpos($table, '_domain_model_')) {
                if($table == 'tx_news_domain_model_news' && $olrow && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']) {
                    DebuggerUtility::var_dump([
                        '$table' => $table,
                        'row['.$languageField.']' => $row[$languageField],
                        '$olrow['.$languageField.']' => $olrow[$languageField],
                        'row' => $row,
                        '$olrow' => $olrow,
                    ],__METHOD__.':'.__LINE__);
                    #die();
                }
                */
                
                
        /*
                if(   $table == 'tx_powermail_domain_model_form'
                   || $table == 'tx_powermail_domain_model_page'
                   || $table == 'tx_powermail_domain_model_field'
                ){
                    
                    if($olrow && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']) {
                        DebuggerUtility::var_dump([
                            '$table' => $table,
                            'row['.$languageField.']' => $row[$languageField],
                            '$olrow['.$languageField.']' => $olrow[$languageField],
                            'row' => $row,
                            '$olrow' => $olrow,
                        ],__METHOD__.':'.__LINE__);
                        #die();
                    }
                    if($row[$languageField] !== $olrow[$languageField]){
                        $row = $olrow;
                        $sys_language_content = $row[ $languageField ];
                    }
                }
                elseif ($olrow && is_array($olrow) && array_key_exists('uid', $olrow) && $olrow['uid'] > 0) {
                    if($row[$languageField] !== $olrow[$languageField]){
                        $row = $olrow;
                        $sys_language_content = $row[ $languageField ];
                    }
                }
                else {
                    
                    #if ($table === 'sys_file_reference') {
                    #    //unset($row);
                    #} else {
                    #    $row = [];
                    #}
                    
                    if(!in_array($row[$languageField], explode(',',$languageChain))){
                         #unset($row);
                         $row = [];
                         $sys_language_content = 0;
                    }
                }
        */
    }

    /**
     * $row as parameter here has already deleted data and is useless in most cases
     * The method is required by the mandatory interface
     */
    public function getRecordOverlay_postProcess($table, &$row, &$sys_language_content, $OLmode, \TYPO3\CMS\Frontend\Page\PageRepository $parent)
    {
        /*
        if(   $table == 'tx_powermail_domain_model_form'
           || $table == 'tx_powermail_domain_model_page'
           || $table == 'tx_powermail_domain_model_field'
        ){
            
            if($row && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']) {
                DebuggerUtility::var_dump([
                    '$table' => $table,
                    #'row['.$languageField.']' => $row[$languageField],
                    #'$olrow['.$languageField.']' => $olrow[$languageField],
                    'row' => $row,
                    #'$olrow' => $olrow,
                ],__METHOD__.':'.__LINE__);
                #die();
            }
        }
        elseif((is_array($row) && (count($row)===0 || $row['uid']=='0')) || $row===false || $row===null){
            unset($row);
            #$sys_language_content = '';
        }
        */
        
        
        
        
        /*
        $tableControl = $GLOBALS['TCA'][$table]['ctrl'] ?? [];
        if($table == 'tx_publikationen_domain_model_publikation') {
            DebuggerUtility::var_dump([
                '$table' => $table,
                'row['.$languageField.']' => $row[$languageField],
                #'$olrow['.$languageField.']' => $olrow[$languageField],
                'row' => $row,
                #'$olrow' => $olrow,
            ],__METHOD__.':'.__LINE__);
            #die();
        }
        */
        
        
        
        /*
        $languageChain = $this->getFallbackChain($parent);
        $languageChainArray = explode(',',$languageChain);
        
        if(!in_array($row['sys_language_uid'], $languageChainArray)){
            // ### DEBUG
            // if($table === 'tx_news_domain_model_news' && $_SERVER['REMOTE_ADDR'] === $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask']){ DebuggerUtility::var_dump(['$table' => $table, '$sys_language_content' => $sys_language_content, '$parent->sys_language_uid' => $parent->sys_language_uid, '$languageChain' => $languageChain, '$row'=> $row, '$olrow'=>$olrow, '$row[sys_language_uid]'=>$row['sys_language_uid'], '$languageChainArray'=>$languageChainArray, ],__METHOD__.':'.__LINE__); }
            
            // `unset($row);` doesn't work like expected with tx_news_domain_model_news but an empty array does.
            $row = [];
        }
        */
    }
    
    protected function getFallbackChain($parent){
        $languageAspect = $parent->context->getAspect('language');
        $fallbackChain = $languageAspect->getFallbackChain();
        // $fallbackChain =>  [0 => 1, 1 => 0, 2 => 'pageNotFound']

        $languageChainArray = [];
        foreach ($fallbackChain as $count => $languageKey) {
            if (MathUtility::canBeInterpretedAsInteger($languageKey)) {
                $languageChainArray[] = $languageKey;
            }
        }
        // TODO: replace $parent->sys_language_uid by site-language
        $languageChain = $parent->sys_language_uid . (count($languageChainArray) ? ',' . implode(',', $languageChainArray) : '');
#DebuggerUtility::var_dump(['$languageChain'=>$languageChain],__METHOD__.':'.__LINE__);
        return $languageChain;
    }
}
